<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKVideo;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Plugin\DataType\EntityAdapter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\field\Entity\FieldConfig;
use Drupal\file\Entity\File;
use Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\node\Entity\Node;
use Drupal\field\FieldConfigInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
/**
 * Provides a 'LBKVideo' block.
 *
 * @Block(
 *  id = "lbk_video",
 *  admin_label = @Translation("Video (LBK)"),
 * )
 */
class LBKVideo extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  protected $currentRouteMatch;

  /**
   * Constructs a new GridComponent object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The ConfigManagerInterface service.
   */
  public function __construct(
      array $configuration,
      $plugin_id,
      $plugin_definition,
      EntityTypeManagerInterface $entity_type_manager,
      ConfigManagerInterface $config_manager,
      CurrentRouteMatch $currentRouteMatch
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configManager = $config_manager;
    $this->currentRouteMatch = $currentRouteMatch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
        $configuration,
        $plugin_id,
        $plugin_definition,
        $container->get('entity_type.manager'),
        $container->get('config.manager'),
        $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'video_component_fields' => [
        'title' => 'Video (LBK)',
        'display_title' => TRUE,
        'video_url' => '',
        'video' => [],
        'video_field' => [],
        'classes' => '',
        'video_radio_options' => [],
        //'videos_embed_array' => [],
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   *
   * Creates a generic configuration form for all block types. Individual
   * block plugins can add elements to this form by overriding
   * BlockBase::blockForm(). Most block plugins should not override this
   * method unless they need to alter the generic form elements.
   *
   * @see \Drupal\Core\Block\BlockBase::blockForm()
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $definition = $this->getPluginDefinition();
    $form['provider'] = [
        '#type' => 'value',
        '#value' => $definition['provider'],
    ];

    /*
    $form['admin_label'] = [
        '#type' => 'item',
        '#title' => $this->t('Block description'),
        '#plain_text' => $definition['admin_label'],
    ];
    */

    // Add context mapping UI form elements.
    $contexts = $form_state->getTemporaryValue('gathered_contexts') ?: [];
    $form['context_mapping'] = $this->addContextAssignmentElement($this, $contexts);
    // Add plugin-specific settings for this block type.
    $form += $this->blockForm($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {

    // Attach library to the Form.
    $form['#attached']['library'] = ['layout_builder_kit/video-styling'];
    $nid = '';

    // Get Node Entity from route.
    $currentRoute = $this->currentRouteMatch;
    $sectionStorage = $currentRoute->getParameters()->get('section_storage');
    if ($sectionStorage instanceof OverridesSectionStorage) {
      $context = $sectionStorage->getContext('entity');
      if ($context instanceof EntityContext) {
        $contextData = $context->getContextData();
        if ($contextData instanceof EntityAdapter) {
          $node = $contextData->getEntity();
          if ($node instanceof Node) {
            $nid = $node->id();
          }
        }
      }
    }

    // Get value from ImageComponentSettingsForm's configuration.
    $configFactory = $this->configManager->getConfigFactory();
    $locationFolderConfig = $configFactory->get('layout_builder_kit.image_component');
    $locationFolder = $locationFolderConfig->get('image_location');

    $form['title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#maxlength' => 255,
        '#default_value' => $this->configuration['video_component_fields']['title'],
        '#required' => TRUE,
        '#weight' => '1',
        '#prefix' => '<div class="custom-video--component">',
    ];

    $form['display_title'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Display title'),
        '#default_value' => $this->configuration['video_component_fields']['display_title'],
        '#weight' => '2',
    ];

    $form['video_radio_options'] = array(
        '#type' => 'radios',
        '#title' => $this->t(''),
        '#default_value' => $this->configuration['video_component_fields']['video_radio_options'],
        '#options' => array(
            0 => $this->t(''),
            //1 => $this->t(''),
            2 => $this->t('')),
        '#id' => 'options_video',
        '#title_display' =>'title',
        '#weight' => '3',
        '#prefix' => '<div class="video-container"><div class="video-radios--options">',
        '#suffix' => '</div>'

    );

    $form['video_url'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Video URL'),
        '#maxlength' => 255,
        '#default_value' => $this->configuration['video_component_fields']['video_url'],
        '#required' => FALSE,
        '#weight' => '4',
        '#prefix' => '<div class="video-fields--options">'
    ];

    /*
      $form['video'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Video'),
      '#upload_location' => ($locationFolder != NULL) ? 'public://' . $locationFolder : 'public://layout_builder_kit',
      '#multiple' => FALSE,
      '#default_value' => $this->configuration['video_component_fields']['video'],
      '#required' => FALSE,
      '#upload_validators' => [
          'file_validate_extensions' => ['jpeg jpg mp4'],
      ],
      '#weight' => '7',
    ];
    */

    $form['video_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Video fields available'),
      '#options' => $this->getVideosEmbedArray($nid),
      '#default_value' => $this->configuration['video_component_fields']['video_field'],
      '#weight' => '5',
      '#suffix' => '</div></div>'
    ];

    $form['classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS class'),
      '#description' => $this->t('A list of CSS classes added to this component separated by spaces.'),
      '#maxlength' => 64,
      '#default_value' => $this->configuration['video_component_fields']['classes'],
      '#weight' => '6',
      '#suffix' => '</div>'
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    $this->configuration['video_component_fields']['title'] = $form_state->getValue('title');
    $this->configuration['video_component_fields']['video_radio_options'] = $form_state->getValue('video_radio_options');
    $this->configuration['video_component_fields']['video_url'] = $form_state->getValue('video_url');
    //$this->configuration['video_component_fields']['video'] = $form_state->getValue('video');
    $this->configuration['video_component_fields']['video_field'] = $form_state->getValue('video_field');
    $this->configuration['video_component_fields']['classes'] = $form_state->getValue('classes');

  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    // Add template.
    $build['#theme'] = 'LBKVideo';
    // Attach library.
    $build['#attached']['library'] = ['layout_builder_kit/video-styling'];

    if (!empty($this->configuration['video_component_fields']['video'])) {
      $imageFileId = implode($this->configuration['video_component_fields']['video']);
      $image = File::load($imageFileId);
      if ($image != NULL) {
        $image->setPermanent();
        $image->save();

        /*
        $build['#video'] = [
            '#theme' => 'image',
            '#uri' => $image->getFileUri(),
        ];
        */
      }
    }

    // TODO: Render field.
    // View builder.
    $entityType = 'node';
    $node = \Drupal::routeMatch()->getParameter('node');

    // Fields to render.
    $fields = [
      $this->configuration['video_component_fields']['video_field']
    ];

    // Get the Node entity object.
    if (\Drupal::routeMatch()->getParameter('node')) {
      $node = \Drupal::routeMatch()->getParameter('node');
      if ($node instanceof Node) {
        $nodeEntity = Node::load($node->id());
      }
    } else {
      $currentRoute = $this->currentRouteMatch;
      $sectionStorage = $currentRoute->getParameters()->get('section_storage');
      if ($sectionStorage instanceof OverridesSectionStorage) {
        $context = $sectionStorage->getContext('entity');
        if ($context instanceof EntityContext) {
          $contextData = $context->getContextData();
          if ($contextData instanceof EntityAdapter) {
            $node = $contextData->getEntity();
            if ($node instanceof Node) {
              $nodeEntity = Node::load($node->id());
            }
          }
        }
      }
    }

    // Get the View for this Node entity object.
    $viewBuilder = $this->entityTypeManager->getViewBuilder($entityType);

    // Build the Node array to render.
    $fieldOutput = [];
    foreach ($fields as $field_name) {
      if(!empty($field_name)) {
        if ($nodeEntity->hasField($field_name) && $nodeEntity->access('view')) {
          $value = $nodeEntity->get($field_name);
          $fieldOutput = $viewBuilder->viewField($value, ["label" => "hidden"]);
          $fieldOutput['#cache']['tags'] = $nodeEntity->getCacheTags();
        }
      }
    }

    $build['#title'] = $this->configuration['video_component_fields']['title'];
    $build['#video_radio_options'] = $this->configuration['video_component_fields']['video_radio_options'];
    //$build['#video_url'] = $this->configuration['video_component_fields']['video_url'];
    $build['#video_url'] = $this->getVideoEmbedUrl($this->configuration['video_component_fields']['video_url']);
    $build['#display_title'] = $this->configuration['video_component_fields']['display_title'];
    $build['#video_field'] = $fieldOutput;
    $build['#classes'] = $this->configuration['video_component_fields']['classes'];

    // To save the field inside this variable.
    //$build['#videos_embed_array'] = $fieldOutput;

    return $build;
  }

  public function getContentTypeFields($contentType) {
    $entityTypeManager = \Drupal::service('entity_field.manager');
    $fields = [];

    if(!empty($contentType)) {
      $fields = array_filter(
          $entityTypeManager->getFieldDefinitions('node', $contentType), function ($field_definition) {
        return $field_definition instanceof FieldConfigInterface;
      }
      );
    }

    return $fields;
  }

  /**
   * @param int $nid
   * @return array
   */
  public function getVideosEmbedArray($nid) {

    $fieldsArray = [];
    $node= Node::load($nid);

    if(!empty($nid)) {
      if ($node instanceof Node) {
        $bundle = $node->bundle();
        $fieldsNode = $this->getContentTypeFields($bundle);

        $fieldsType = [];
        foreach($fieldsNode as $field) {
          if ($field instanceof FieldConfig) {

            $fieldsType[$field->getName()] = $field->getType();
            unset($fieldsType['layout_builder__layout']);

            /*
            if($field->getName() !== 'layout_builder__layout') {
              $fieldsArray[$field->getName()] = $field->getLabel();
            }
            */

            //Filtering by field type

            switch ($field->getType()) {
              case "string":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;
              case "video_embed_field":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;
              case "link":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;
              case "file":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;
            }
          }
        }
      }
    }

    return $fieldsArray;
  }

  /**
   * @param string $videoUrl
   * @return string
   */
  public function getVideoEmbedUrl($videoUrl){

    $finalVideoUrl = '';

    if(strpos($videoUrl, 'vimeo.com/') !== false) {

      //it is Vimeo video
      $videoId = explode("vimeo.com/",$videoUrl)[1];
      if(strpos($videoId, '&') !== false){
        $videoId = explode("&",$videoId)[0];
      }
      $finalVideoUrl.='https://player.vimeo.com/video/'.$videoId;
    } else if(strpos($videoUrl, 'youtube.com/watch') !== false) {

      //it is Youtube video
      $videoId = explode("v=",$videoUrl)[1];
      if(strpos($videoId, '&') !== false){
        $videoId = explode("&",$videoId)[0];
      }
      $finalVideoUrl.='https://www.youtube.com/embed/'.$videoId;
    } else if(strpos($videoUrl, 'youtu.be/') !== false){

      //it is Youtube video
      $videoId = explode("youtu.be/",$videoUrl)[1];
      if(strpos($videoId, '&') !== false){
        $videoId = explode("&",$videoId)[0];
      }
      $finalVideoUrl.='https://www.youtube.com/embed/'.$videoId;
    } else if(strpos($videoUrl, 'youtube.com/embed') !== false){

      //It is a valid Youtube embed video
      $finalVideoUrl.= $videoUrl;

    } else {
      //It's not a valid video URL
    }

    //$urlParts   = explode('/', $videoUrl);
    //$vidid      = explode( '&', str_replace('watch?v=', '', end($urlParts) ) );
    //return 'https://www.youtube.com/embed/' . $vidid[0] ;

    return $finalVideoUrl;
  }
}
